﻿using Magelia.WebStore.Contracts;
using Magelia.WebStore.Services.Contract.Parameters.Store;
using Orchard;
using Orchard.Security;
using Orchard.Users.Events;
using System;
using System.Globalization;

namespace Magelia.WebStore.Handlers
{
    public class UserEventHandler : IUserEventHandler
    {
        private IUserModelsStateServices _userModelsStateServices;
        private IWebStoreServices _webStoreServices;
        private IOrchardServices _orchardServices;

        private void TransfertAndUpdateBasket(String fromUserName, String newUsername)
        {
            this._webStoreServices.UsingClient(
                wc =>
                {
                    wc.StoreClient.TransferBaskets(fromUserName, newUsername, new[] { "default" });
                    wc.StoreClient.UpdateBasket(newUsername, "default", this._webStoreServices.CurrentCurrencyId, CultureInfo.GetCultureInfo(this._orchardServices.WorkContext.CurrentCulture).LCID, new Location { CountryId = this._webStoreServices.CurrentCountryId, RegionId = this._webStoreServices.CurrentRegionId });
                }
            );
        }

        public UserEventHandler(IWebStoreServices webStoreServices, IOrchardServices orchardServices, IUserModelsStateServices userModelsStateServices)
        {
            this._userModelsStateServices = userModelsStateServices;
            this._webStoreServices = webStoreServices;
            this._orchardServices = orchardServices;
        }

        public void Creating(UserContext context)
        {

        }

        public void Created(UserContext context)
        {
            this._webStoreServices.EnsureUser(context.User);
        }

        public void LoggedIn(IUser user)
        {
            this._userModelsStateServices.FlushUserContext();
            this._webStoreServices.EnsureUser(user);
            this.TransfertAndUpdateBasket(this._webStoreServices.AnonymousUserName, user.UserName);
        }

        public void LoggedOut(IUser user)
        {
            this._userModelsStateServices.FlushUserContext();
            this.TransfertAndUpdateBasket(user.UserName, this._webStoreServices.AnonymousUserName);
        }

        public void AccessDenied(IUser user)
        {

        }

        public void ChangedPassword(IUser user)
        {

        }

        public void SentChallengeEmail(IUser user)
        {

        }

        public void ConfirmedEmail(IUser user)
        {

        }

        public void Approved(IUser user)
        {

        }
    }
}